import os
from pathlib import Path
import shutil
import random

MOCK_RANDOM_FIX = 11110


def random_5_digit_number():
    if os.getenv("MOCK") == "yes":
        global MOCK_RANDOM_FIX
        MOCK_RANDOM_FIX = MOCK_RANDOM_FIX + 1
        return MOCK_RANDOM_FIX
    else:
        return random.randint(10000, 99999)


class TempDir:
    """
    Handle temporary directory and files
    """

    def __init__(self):
        self.orjitmp = Path(os.getenv("ORJITMP", "."))

    def create(self):
        assert self.orjitmp.exists()
        assert self.orjitmp.is_dir()
        self.working_dir = self.orjitmp.joinpath(
            f"{random_5_digit_number()}.tmp"
        ).absolute()
        self.working_dir.mkdir()

    def tempfile(self, text, filename=None):
        if filename is None:
            filename = f"{random_5_digit_number()}.txt"
        filepath = Path(f"{self.working_dir}/{filename}").absolute()
        filepath.write_text(text)
        return filepath

    def destroy(self):
        shutil.rmtree(self.working_dir.absolute())



from pathlib import Path
from orgmunge import ORG_TIME_FORMAT
import icalendar
import jinja2

DEFAULT = jinja2.Template(
    """\
{% for event in events %}* {{ event.summary }}
SCHEDULED: <{{ event.scheduled }}>

{{ event.description }}
{% endfor %}"""
)


class Event:
    def __init__(self, ievent):
        self._ievent = ievent

    @property
    def summary(self):
        return self._ievent.get("SUMMARY")

    @property
    def description(self):
        return self._ievent.get("DESCRIPTION")

    @property
    def scheduled(self):
        return (
            self._ievent.get("DTSTAMP")
            .from_ical(self._ievent.get("DTSTAMP"))
            .strftime(ORG_TIME_FORMAT)
        )


class ICal:
    def __init__(self, icalfile):
        self._icalfile = icalfile
        self._icaltext = Path(icalfile).read_text()
        self._ical = icalendar.Calendar.from_ical(self._icaltext)
        self._events = [
            Event(ievent)
            for ievent in self._ical.walk()
            if ievent.get("SUMMARY") is not None
        ]

    def to_org(self):
        return DEFAULT.render(events=self._events)



import click
from .tempdir import TempDir
from .lookup import Lookup
from .loader import Loader
from pathlib import Path


@click.command()
@click.argument("location")
@click.option(
    "--children",
    "children",
    is_flag=True,
    help="Remove all children of referenced note, not the actual note.",
)
def remove(location, children):
    temp_dir = TempDir()
    temp_dir.create()
    loader = Loader(temp_dir)
    lookup = Lookup(location)
    item = lookup.load(loader)
    if children:
        item.delete_children()
    else:
        item.delete()
    Path(lookup.filepath).write_text(str(item._org))
    click.echo("Deleted note(s) successfully")
    temp_dir.destroy()



from hitchstory import (
    StoryCollection,
    BaseEngine,
    exceptions,
    validate,
    no_stacktrace_for,
    strings_match,
    Failure,
)
from hitchstory import GivenDefinition, GivenProperty, InfoDefinition, InfoProperty
from templex import Templex
from strictyaml import Optional, Str, Map, Int, Bool, Enum, load, MapPattern, EmptyDict
from path import Path
import hitchpylibrarytoolkit
from hitchrunpy import (
    ExamplePythonCode,
    HitchRunPyException,
    ExpectedExceptionMessageWasDifferent,
)
from shlex import split
from templex import Templex
from commandlib import Command


class Engine(BaseEngine):
    """Python engine for running tests."""

    given_definition = GivenDefinition(
        files=GivenProperty(
            MapPattern(Str(), Str()),
            inherit_via=GivenProperty.OVERRIDE,
        ),
    )

    info_definition = InfoDefinition(
        status=InfoProperty(schema=Enum(["experimental", "stable"])),
        docs=InfoProperty(schema=Str()),
    )

    def __init__(self, keypath, python_path=None, rewrite=False, cprofile=False):
        self.path = keypath
        self._python_path = python_path
        self._rewrite = rewrite
        self._cprofile = cprofile

    def set_up(self):
        """Set up your applications and the test environment."""
        self.path.profile = self.path.gen.joinpath("profile")
        self.path.working = self.path.gen.joinpath("working")

        if self.path.working.exists():
            self.path.working.rmtree()
        self.path.working.mkdir()
        self.path.working.joinpath("tmp").mkdir()

        for filename, contents in self.given["files"].items():
            filepath = self.path.working.joinpath(filename)
            if not filepath.dirname().exists():
                filepath.dirname().mkdir()
            self.path.working.joinpath(filename).write_text(contents)

        if not self.path.profile.exists():
            self.path.profile.mkdir()

        self.python = Command(self._python_path)
        self.orji_bin = Command(self._python_path.parent / "orji").with_env(MOCK="yes")

    @no_stacktrace_for(AssertionError)
    @validate(
        cmd=Str(),
        output=Str(),
        env=MapPattern(Str(), Str()) | EmptyDict(),
        error=Bool(),
    )
    def orji(self, cmd, output, env=None, error=False):
        env = {} if env is None else env
        command = self.orji_bin(*split(cmd)).with_env(**env).in_dir(self.path.working)

        if error:
            command = command.ignore_errors()

        actual_output = command.output()

        try:
            strings_match(output, actual_output)
        except Failure:
            if self._rewrite:
                self.current_step.update(output=actual_output)
            else:
                raise

    @no_stacktrace_for(AssertionError)
    def file_contents(self, filename, contents):
        filepath = self.path.working.joinpath(filename)
        assert filepath.exists()
        try:
            strings_match(contents, filepath.text())
        except Failure:
            if self._rewrite:
                self.current_step.update(contents=filepath.text())
            else:
                raise

    @no_stacktrace_for(AssertionError)
    def pdf(self, cmd):
        output = self.orji_bin(*split(cmd)).in_dir(self.path.working).output()
        self.path.working.joinpath("latex.tex").write_text(output)
        from commandlib import Command

        self.path.working.chdir()
        # import IPython ; IPython.embed()
        Command("pdflatex", "latex.tex").in_dir(self.path.working).run()

    def pause(self, message="Pause"):
        import IPython

        IPython.embed()

    def on_success(self):
        if self._rewrite:
            self.new_story.save()
        if self._cprofile:
            self.python(
                self.path.key.joinpath("printstats.py"),
                self.path.profile.joinpath("{0}.dat".format(self.story.slug)),
            ).run()



from .exceptions import OrjiError, Failure
import jinja2
import click
import traceback
from sys import exit
from pathlib import Path
import inspect
import importlib.machinery
import importlib.util


def fail(message):
    raise Failure(message)


def environment(latexmode, pymodule_filename):
    if latexmode:
        env = jinja2.Environment(
            block_start_string=r"\BLOCK{",
            block_end_string="}",
            variable_start_string=r"\VAR{",
            variable_end_string="}",
            comment_start_string=r"\#{",
            comment_end_string="}",
            line_statement_prefix="%%",
            line_comment_prefix="%#",
            trim_blocks=True,
            autoescape=False,
            undefined=jinja2.StrictUndefined,
            loader=jinja2.BaseLoader,
        )
    else:
        env = jinja2.Environment(
            undefined=jinja2.StrictUndefined, loader=jinja2.BaseLoader
        )

    if pymodule_filename is not None:
        pymodule_filepath = Path(pymodule_filename)

        if not pymodule_filepath.exists():
            click.echo(f"{pymodule_filename} not found", err=True)
            exit(1)

        loader = importlib.machinery.SourceFileLoader(
            pymodule_filepath.stem, str(pymodule_filepath.absolute())
        )
        spec = importlib.util.spec_from_loader(loader.name, loader)
        pymodule = importlib.util.module_from_spec(spec)
        loader.exec_module(pymodule)

        module_contents = {
            key: item
            for key, item in inspect.getmembers(pymodule)
            if not key.startswith("_")
        }
        env.globals.update(module_contents)

    env.globals["fail"] = fail
    return env


class Template:
    def __init__(
        self, template_text, jinjafile, latexmode=None, pymodule_filename=None
    ):
        self._template_text = template_text
        self._jinjafile = jinjafile
        self._latexmode = latexmode
        self._pymodule_filename = pymodule_filename
        self._env = environment(latexmode, pymodule_filename)

    def render(self, **vars):
        try:
            return self._env.from_string(self._template_text).render(**vars)
        except jinja2.exceptions.UndefinedError as error:
            lineno = traceback.extract_tb(error.__traceback__)[-1].lineno
            click.echo(
                f"Template error on line {lineno} of {self._jinjafile}: {error}",
                err=True,
            )
            exit(1)
        except jinja2.exceptions.TemplateSyntaxError as error:
            lineno = traceback.extract_tb(error.__traceback__)[-1].lineno
            click.echo(
                f"Template syntax error on line {lineno} of {self._jinjafile}: {error}",
                err=True,
            )
            exit(1)
        except jinja2.exceptions.TemplateRuntimeError as error:
            lineno = traceback.extract_tb(error.__traceback__)[-1].lineno
            click.echo(
                f"Template runtime error on line {lineno} of {self._jinjafile}: {error}",
                err=True,
            )
            exit(1)
        except Failure as error:
            lineno = traceback.extract_tb(error.__traceback__)[-2].lineno
            click.echo(
                f"Failure on line {lineno} of {self._jinjafile}: {error}", err=True
            )
            exit(1)
        except OrjiError as error:
            lineno = traceback.extract_tb(error.__traceback__)[-3].lineno
            click.echo(
                f"Failure on line {lineno} of {self._jinjafile}: {error}", err=True
            )
            exit(1)



from slugify import slugify
from .exceptions import OrjiError
from .lookup import Lookup
import re


class NoteGroup:
    def __init__(self, notes):
        self._notes = notes

    def walk(self):
        all_notes = [note.walk() for note in self._notes]
        return [subnote for note in all_notes for subnote in note]


class TextChunk:
    def __init__(self, text):
        self.text = str(text) if text is not None else ""

    @property
    def markdown(self):
        text = self.text
        text = re.sub(
            re.compile(r"\[\[(.*?)\]\[(.*?)\]\]"),
            r"[\2](\1)",
            text,
        )
        text = text.replace("\n+ ", "\n* ")
        return text.strip()

    @property
    def latexed(self):
        text = self.text
        text = re.sub(
            re.compile(r"\[\[(.*?)\]\[(.*?)\]\]"),
            r"\\href{\1}{\2}",
            text,
        )
        text = text.replace("&", "\\&")
        return text.strip()

    @property
    def strip(self):
        return self.text.strip()

    def __str__(self):
        return self.text.strip()


class Links:
    def __init__(self, note):
        self._note = note

    def _grab(self, link_type, link_name):
        matching = list(
            re.findall(link_type + r"\:(.*)?(?:\s|$)", self._note.body.text)
        )
        if len(matching) == 0:
            raise OrjiError(f"No {link_name} detected")
        elif len(matching) > 1:
            raise OrjiError(f"Over one ({len(matching)}) {link_name} detected")
        else:
            return matching[0]

    @property
    def tel(self):
        return self._grab(r"tel", "telephone numbers")

    @property
    def sms(self):
        return self._grab(r"sms", "SMS numbers")

    @property
    def file(self):
        return self._grab(r"file", "file (file:) links")

    @property
    def mailto(self):
        return self._grab(r"mailto", "mailto (mailto:) links")


class Body(TextChunk):
    def __init__(self, text, temp_dir):
        self.text = str(text) if text is not None else ""
        self._temp_dir = temp_dir

    @property
    def oneline(self):
        if "\n" not in self.text.strip():
            return self.text.strip()
        else:
            raise OrjiError(f"{self.text} is not one line")

    def tempfile(self):
        return self._temp_dir.tempfile(self.text)

    @property
    def paragraphs(self):
        return [
            TextChunk(text) for text in self.text.split("\n\n") if text.strip() != ""
        ]


class Note:
    def __init__(self, node, loader, org):
        self._node = node
        self._loader = loader
        self._org = org

    @property
    def name(self):
        return self._node.headline.title

    @property
    def links(self):
        return Links(self)

    @property
    def indexlookup(self):
        indices = []
        node = self._node
        while True:
            index = [i for i, n in enumerate(node.parent.children) if n == node][0]
            indices.append(str(index))
            if node.parent.parent is None:
                break
            else:
                node = node.parent

        return "/".join(reversed(indices))

    @property
    def slug(self):
        return slugify(self._node.headline.title)

    @property
    def state(self):
        return self._node.todo

    @property
    def tags(self):
        unsorted = self._node.tags
        return sorted(unsorted) if unsorted is not None else []

    @property
    def body(self):
        return Body(self._node.body, self._loader._temp_dir)

    @property
    def prop(self):
        return self._node.properties

    def from_indexlookup(self, indexlookup):
        split = [int(x) for x in indexlookup.split("/")]
        node = self._node

        for index in split:
            node = node.children[index]

        return Note(node, self._loader, self._rg)

    def delete(self):
        parent = self._node.parent
        node_index = [
            i for i, node in enumerate(parent.children) if node == self._node
        ][0]
        del parent.children[node_index]

    def delete_children(self):
        self._node.children = []

    def has(self, lookup):
        return Lookup(lookup, relative_to=self).exists(loader=self._loader)

    def at(self, lookup):
        return Lookup(lookup, relative_to=self).load(loader=self._loader)

    def __str__(self):
        return str(self._org)

    def _walk_iter(self, node):
        if len(node.children) == 0:
            return [Note(node, self._loader, self._org)]
        children = []
        for child in node.children:
            children.extend(self._walk_iter(child))
        return children

    def walk(self):
        return self._walk_iter(self._node)

    @property
    def children(self):
        return [Note(node, self._loader, self._org) for node in self._node.children]

    def __iter__(self):
        for node in self._node.children:
            yield Note(node, self._loader, self._org)



from hitchstory import StoryCollection
from pathquery import pathquery
from click import argument, group, pass_context
import hitchpylibrarytoolkit
from engine import Engine


toolkit = hitchpylibrarytoolkit.ProjectToolkitV2(
    "OrJi",
    "orji",
    "crdoconnor/orji",
    image="",
)


@group(invoke_without_command=True)
@pass_context
def cli(ctx):
    """Integration test command line interface."""
    pass


DIR = toolkit.DIR


def _storybook(**settings):
    return StoryCollection(pathquery(DIR.key).ext("story"), Engine(DIR, **settings))


def _current_version():
    return DIR.project.joinpath("VERSION").text().rstrip()


def _devenv():
    return toolkit.devenv()


@cli.command()
@argument("keywords", nargs=-1)
def rbdd(keywords):
    """
    Run story with name containing keywords and rewrite.
    """
    _storybook(python_path=_devenv().python_path, rewrite=True).shortcut(
        *keywords
    ).play()


@cli.command()
@argument("keywords", nargs=-1)
def bdd(keywords):
    """
    Run story with name containing keywords.
    """
    _storybook(python_path=_devenv().python_path).shortcut(*keywords).play()


@cli.command()
@argument("filename")
def regressfile(filename):
    """
    Run all stories in filename 'filename'.
    """
    StoryCollection(
        pathquery(DIR.key).ext("story"), Engine(DIR, python_path=_devenv().python_path)
    ).in_filename(filename).ordered_by_name().play()


@cli.command()
def rewriteall():
    """
    Run all stories in rewrite mode.
    """
    StoryCollection(
        pathquery(DIR.key).ext("story"),
        Engine(DIR, python_path=_devenv().python_path, rewrite=True),
    ).only_uninherited().ordered_by_name().play()


@cli.command()
def regression():
    """
    Continuos integration - lint and run all stories.
    """
    # toolkit.lint(exclude=["__init__.py"])
    StoryCollection(
        pathquery(DIR.key).ext("story"), Engine(DIR, python_path=_devenv().python_path)
    ).only_uninherited().ordered_by_name().play()


@cli.command()
def reformat():
    """
    Reformat using black and then relint.
    """
    toolkit.reformat()


@cli.command()
def lint():
    """
    Lint project code and hitch code.
    """
    toolkit.lint(exclude=["__init__.py"])


@cli.command()
@argument("test", required=False)
def deploy(test="test"):
    """
    Deploy to pypi as specified version.
    """
    testpypi = not (test == "live")
    toolkit.deploy(testpypi=testpypi)


@cli.command()
def draftdocs():
    """
    Build documentation.
    """
    toolkit.draft_docs(storybook=_storybook(python_path=_devenv().python_path))


@cli.command()
def publishdocs():
    """Publish pushed docs."""
    toolkit.publish(storybook=_storybook(python_path=_devenv().python_path))


@cli.command()
def build():
    _devenv()


@cli.command()
def cleandevenv():
    DIR.gen.joinpath("pyenv", "versions", "devvenv").remove()


@cli.command()
@argument("strategy_name", nargs=1)
def envirotest(strategy_name):
    """Run tests on package / python version combinations."""
    import envirotest
    import pyenv

    test_package = pyenv.PythonRequirements(
        [
            "prettystack=={}".format(_current_version()),
        ],
        test_repo=True,
    )

    test_package = pyenv.PythonProjectDirectory(DIR.project)

    prerequisites = [
        pyenv.PythonVersionDependentRequirement(
            package="markupsafe",
            lower_version="2.0.0",
            python_version_threshold="3.9",
            higher_version="2.1.2",
        ),
        pyenv.PythonRequirements(
            [
                "ensure",
            ]
        ),
    ]

    envirotest.run_test(
        pyenv.Pyenv(DIR.gen / "pyenv"),
        DIR.project.joinpath("pyproject.toml").text(),
        test_package,
        prerequisites,
        strategy_name,
        _storybook,
        lambda python_path: False,
    )


if __name__ == "__main__":
    cli()



from .note import Note
from pathlib import Path
import click
from click import echo
from sys import exit
import stat
import subprocess
from .template import Template
import orgmunge
from .tempdir import TempDir
from .loader import Loader


@click.command()
@click.argument("orgdir")
@click.argument("rundir")
@click.option(
    "--out",
    help="Output folder. Defaults to current.",
)
@click.option(
    "--multiple/--single",
    help="Output folder. Defaults to current.",
    default=False,
)
def run(orgdir, rundir, out, multiple):
    temp_dir = TempDir()
    temp_dir.create()
    loader = Loader(temp_dir)
    orgdir = Path(orgdir).absolute()
    rundir = Path(rundir).absolute()

    assert orgdir.is_dir()
    assert rundir.is_dir()
    assert len(list(orgdir.glob("*.org"))), "orgdir must contain org files"
    assert len(list(rundir.glob("*.sh"))), "rundir must contain sh files"

    scripts = {script.stem: script.read_text() for script in rundir.glob("*.sh")}

    out_dir = Path("." if out is None else out).absolute()

    assert out_dir.is_dir()
    assert out_dir.exists()

    matching_notes = []

    for orgfile in orgdir.glob("*.org"):
        parsed_munge = orgmunge.Org(
            Path(orgfile).read_text(),
            from_file=False,
            todos={"todo_states": {"todo": "TODO"}, "done_states": {"done": "DONE"}},
        )

        for note in Note(parsed_munge.root, loader=loader, org=parsed_munge):
            if note.state == "TODO":
                for tag in note.tags:
                    if tag in scripts.keys():
                        matching_notes.append((orgfile, tag, note))

    if len(matching_notes) == 0 and not multiple:
        temp_dir.destroy()
        echo("No scripts were run")
        exit(1)
    elif len(matching_notes) > 1 and not multiple:
        temp_dir.destroy()
        echo("Multiple matching notes use --multiple to run all of them")
        echo("")
        for orgfile, _, note in matching_notes:
            echo(f"{orgfile}: {note.indexlookup}: {note.name}")
        exit(1)
    else:
        for orgfile, tag, note in matching_notes:
            rendered_script = Template(scripts[tag], f"{tag}.sh").render(
                notebody=temp_dir.tempfile(str(note.body), filename="notebody.txt"),
                orgfile=orgfile,
                note=note,
                tmp=temp_dir.working_dir,
                out=out_dir,
                rundir=rundir,
                orgdir=orgdir,
            )
            tmp_script = temp_dir.tempfile(rendered_script, filename=f"{tag}.sh")
            tmp_script.chmod(tmp_script.stat().st_mode | stat.S_IEXEC)
            return_code = subprocess.call(["bash", "-e", tmp_script])
            if return_code != 0:
                print(f"\n\nERROR running {tag}.sh in {temp_dir.working_dir}")
                exit(return_code)
        temp_dir.destroy()



from pathlib import Path
from .note import Note, NoteGroup
from .exceptions import OrjiError
import orgmunge


class Loader:
    def __init__(self, temp_dir, multiple_files_ok=False):
        self._temp_dir = temp_dir
        self._multiple_files_ok = multiple_files_ok

    def load(self, filepath: str):
        filepath_obj = Path(filepath)
        if filepath_obj.is_dir():
            if self._multiple_files_ok:
                notes = []
                for orgfile in filepath_obj.glob("*.org"):
                    org_text = orgfile.read_text()
                    munge_parsed = orgmunge.Org(
                        org_text,
                        from_file=False,
                        todos={
                            "todo_states": {"todo": "TODO"},
                            "done_states": {"done": "DONE"},
                        },
                    )
                    notes.append(Note(munge_parsed.root, loader=self, org=munge_parsed))
                return NoteGroup(notes)
            else:
                raise OrjiError(f"{filepath} is a directory.")
        else:
            org_text = Path(filepath).read_text()
            munge_parsed = orgmunge.Org(
                org_text,
                from_file=False,
                todos={
                    "todo_states": {"todo": "TODO"},
                    "done_states": {"done": "DONE"},
                },
            )
            return Note(munge_parsed.root, loader=self, org=munge_parsed)



from enum import Enum
from .exceptions import OrjiError


class LookupType(Enum):
    FILE = 0
    ABSOLUTE = 1
    RELATIVE = 2
    FILEONLY = 3


class LookupItemType(Enum):
    INDEX = 0
    NAME = 1


class LookupItem:
    def __init__(self, item: str):
        if item.isdigit():
            self.item_type = LookupItemType.INDEX
            self.index = int(item)
        elif item.startswith("'") and item.endswith("'"):
            self.name = item.strip("'")
            self.item_type = LookupItemType.NAME
        else:
            self.item_type = LookupItemType.NAME
            self.name = item


class Lookup:
    def __init__(self, text, relative_to=None):
        self.relative_to = relative_to

        if "//" in text:
            self.lookup_type = LookupType.FILE
            split = text.split("//")
            self.filepath = split[0]
            self.ref = split[1]
            self.parsed_ref = [LookupItem(item) for item in self.ref.split("/")]
        elif text.startswith("./"):
            self.lookup_type = LookupType.RELATIVE
            raise NotImplementedError
        elif text.startswith("/"):
            self.lookup_type = LookupType.ABSOLUTE
            raise NotImplementedError
        else:
            if relative_to is None:
                self.lookup_type = LookupType.FILEONLY
                self.filepath = text
                self.parsed_ref = []
                self.ref = None
            else:
                self.lookup_type = LookupType.RELATIVE
                self.ref = text
                self.parsed_ref = [LookupItem(item) for item in text.split("/")]

    def exists(self, loader):
        return self.load(loader, fail_if_nonexistent=False) is not None

    def load(self, loader, fail_if_nonexistent=True):
        if self.relative_to is None:
            current_note = loader.load(self.filepath)
        else:
            current_note = self.relative_to
        for item in self.parsed_ref:
            if item.item_type == LookupItemType.INDEX:
                current_note = current_note.children[item.index]
            else:
                matching_notes = [
                    note for note in current_note.children if note.name == item.name
                ]
                if len(matching_notes) == 1:
                    return matching_notes[0]
                elif len(matching_notes) > 1:
                    raise OrjiError(f"More than one note found matching '{self.ref}'")
                else:
                    if fail_if_nonexistent:
                        raise OrjiError(f"No notes matching '{self.ref}' found.")
                    else:
                        return None
        return current_note



from pathlib import Path
import click
from .template import Template
from .tempdir import TempDir
from .lookup import Lookup
from .loader import Loader
from .note import Note


@click.command()
@click.argument("orglookup")
@click.argument("jinjafile")
@click.option(
    "--latexmode",
    is_flag=True,
    show_default=True,
    default=False,
    help="Jinja2 latex mode",
)
@click.option(
    "--module",
    "pymodule",
    help="Specify python module to use in template.",
)
def output(orglookup, jinjafile, latexmode, pymodule):
    lookup = Lookup(orglookup)
    temp_dir = TempDir()
    temp_dir.create()
    loader = Loader(temp_dir, multiple_files_ok=True)
    template_text = Path(jinjafile).read_text()
    notes = lookup.load(loader)

    if isinstance(notes, Note):
        output_text = Template(
            template_text, jinjafile, latexmode=latexmode, pymodule_filename=pymodule
        ).render(note=notes, notes=notes, root=notes, at=notes.at, has=notes.has)
    else:
        output_text = Template(
            template_text, jinjafile, latexmode=latexmode, pymodule_filename=pymodule
        ).render(orgfiles=notes)

    click.echo(output_text)
    temp_dir.destroy()



from pathlib import Path
import click
from .template import Template
import orgmunge
from .tempdir import TempDir
from .loader import Loader
from .lookup import Lookup
from .ical import ICal


@click.command()
@click.argument("jinjafile")
@click.argument("relative")
@click.argument("location")
@click.option(
    "--text",
    "textfile",
    help="Put file contents into {{ text }}",
)
@click.option(
    "--ical",
    "icalfile",
    help="Put ical file contents into {{ ical }}",
)
def insert(jinjafile, relative, location, textfile, icalfile):
    temp_dir = TempDir()
    temp_dir.create()
    loader = Loader(temp_dir)
    lookup = Lookup(location)
    template_text = Path(jinjafile).read_text()
    output_text = Template(template_text, jinjafile).render(
        text=Path(textfile).read_text() if textfile is not None else None,
        ical=ICal(icalfile) if icalfile is not None else None,
    )
    chunk_to_insert = orgmunge.Org(
        output_text,
        from_file=False,
        todos={"todo_states": {"todo": "TODO"}, "done_states": {"done": "DONE"}},
    )

    write_note = lookup.load(loader)

    assert len(chunk_to_insert.root.children) == 1
    if relative == "above":
        for note in chunk_to_insert.root.children:
            if write_note._node.sibling is None:
                write_note._org.root.add_child(note)
            else:
                write_note._node.sibling.add_child(note)
                note.sibling = write_note._node.sibling
                note.demote()
    elif relative == "below":
        for note in chunk_to_insert.root.children:
            write_note._node.add_child(note)

            for _ in range(write_note._node.level - 1):
                note.sibling = write_note._node
                note.demote()
    elif relative == "under":
        for note in chunk_to_insert.root.children:
            write_note._node.parent.add_child(note)

            for _ in range(write_note._node.level):
                note.sibling = write_note._node
                note.demote()
    elif relative == "replace":
        for note in chunk_to_insert.root.children:
            write_note._node.sibling.add_child(note)
            note.sibling = write_note._node.sibling
            note.demote()
            parent = write_note._node.parent
            node_index = [
                i for i, node in enumerate(parent.children) if node == write_note._node
            ][0]
            del parent.children[node_index]
    else:
        raise NotImplementedError("f{relative} not implemented")
    Path(lookup.filepath).write_text(str(write_note))
    click.echo("Written note(s) successfully")
    temp_dir.destroy()



from .output import output
from .run import run
from .insert import insert
from .remove import remove
import click


@click.group()
def cli():
    pass


cli.add_command(output, name="out")
cli.add_command(run)
cli.add_command(insert, name="in")
cli.add_command(remove, name="rm")



from strictyaml import load
from commandlib import Command, python_bin
import hitchpylibrarytoolkit


README_INTRO = """# OrJi

[![Main branch status](https://github.com/crdoconnor/orji/actions/workflows/regression.yml/badge.svg)](https://github.com/crdoconnor/orji/actions/workflows/regression.yml)"""

DOCS_INTRO = """---
title: OrJi
---

<a href="https://github.com/crdoconnor/orji"><img alt="GitHub Repo stars" src="https://img.shields.io/github/stars/crdoconnor/orji?style=social"/></a>

<a href="https://pypi.org/pypi/orji"><img alt="PyPI - Downloads" src="https://img.shields.io/pypi/dm/orji"/></a>
"""


def _title(filepath):
    try:
        assert len(filepath.text().split("---")) >= 3, "{} doesn't have ---".format(
            filepath
        )
        return load(filepath.text().split("---")[1]).data.get("title", "misc")
    except UnicodeDecodeError:
        return None
    except AssertionError:
        return None


def _contents(main_folder, folder, readme):
    markdown = ""
    for filepath in sorted(main_folder.joinpath(folder).listdir()):
        if filepath.name != "index.md":
            title = _title(filepath)

            if title is not None:
                path = filepath.relpath(main_folder).stripext()

                markdown += "- [{}]({})\n".format(
                    title,
                    "https://hitchdev.com/orji/" + path if readme else path,
                )
    return markdown


def run_docgen(paths, storybook, publish=False, readme=False):
    dirtempl = python_bin.dirtempl.in_dir(paths.project / "docs")
    doc_src = paths.project / "docs" / "src"
    snippets_path = paths.project / "docs" / "snippets"
    git = Command("git").in_dir(paths.project)

    if publish:
        dest_path = paths.gen / "orji" / "docs" / "public"
    else:
        dest_path = paths.project / "docs" / "draft"

    if snippets_path.exists():
        snippets_path.rmtree()
    snippets_path.mkdir()

    if dest_path.exists():
        dest_path.rmtree()

    snippets_path.joinpath("quickstart.txt").write_text(
        storybook.with_documentation(
            paths.key.joinpath("docstory.yml").text(),
            extra={"title": False},
        )
        .named("Quickstart")
        .documentation()
    )

    generate_storydocs(
        paths.key.joinpath("docstory.yml").text(),
        doc_src.joinpath("using"),
        storybook,
    )

    snippets_path.joinpath("intro.txt").write_text(
        README_INTRO if readme else DOCS_INTRO
    )

    for folder in ["using"]:
        snippets_path.joinpath(f"{folder}-contents.txt").write_text(
            _contents(doc_src, folder, readme=readme)
        )

    for folder in ["using"]:
        snippets_path.joinpath(f"{folder}-index-contents.txt").write_text(
            _contents(doc_src / folder, "", readme=readme)
        )

    dirtempl("--snippets", snippets_path, doc_src, dest_path).run()

    dest_path.joinpath("changelog.md").write_text(
        hitchpylibrarytoolkit.docgen.changelog(paths.project)
    )


def generate_storydocs(docstory, docpath, storybook):
    storydocs = storybook.with_documentation(
        docstory,
        extra={"title": True},
    )

    for story in storydocs.ordered_by_file():
        docfilename = story.info.get("docs")
        if docfilename is not None:
            docpath.joinpath(docfilename + ".md").write_text(story.documentation())
